      subroutine calc_rects(pivot,fibre,fibdims,button,tube)
C+------------------------------------------------------------------------------
C
C     Name:   
C        calc_rects
C
C     Invocation:
C        call calc_rects(pivot,fibre,fibdims,button,tube)
C
C     Description:
C        The rectangles which define the limits of a fibre tube and button
C        are calculated.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        pivot:  Real array (>)
C           The x,y coordinates of the fibre pivot.
C        fibre:  Real array (>)
C           The x,y coordinates of the fibre button.
C        fibdims:  Integer array (>)
C           The fibre button length, button width and tube width.
C        button:  Real array (<)
C           The x,y coordinates of the rectangle defined by the
C           fibre button. The first four coordinates are x, the
C           second four are y.
C        tube:  Real array (<)
C           The x,y coordinates of the rectangle defined by the
C           fibre tube. The first four coordinates are x, the
C           second four are y.
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        none
C 
C     Algorithm:
C        -- The projection factors are calculated for the position angle
C           of the fibre.
C        -- The x,y coordinates of the button are calculated based on the
C           idea that the object is placed a distance of half the width
C           of the button perpendicular to the button end.
C        -- The x,y coordinates of the tube are calculated from the pivot
C           to the object location.
C        -- The coordinates are stored in a circular manner so that the
C           plotting routine will draw a closed figure.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      real pivot(2)
      real fibre(2)
      integer fibdims(3)
      real button(8)
      real tube(8)
C
C     Local variables
C
      real dx,dy,rad,costheta,sintheta,rblen,hbw,htw,bl1,bl2,tmp
C
C     Get the trigonometric functions we need for the projections
C
      dx = fibre(1) - pivot(1)
      dy = fibre(2) - pivot(2)
      rad = sqrt(dx*dx + dy*dy)
      costheta = dx/rad
      sintheta = dy/rad
C
C     Now solve for the x,y positions of the corners of the button.  X
C     positions are in the first four elements of 'button', y positions are
C     in the second four.  The position of the corners is based on the idea
C     that the object is located one half a button width from the end of
C     the button.  Note that we define the button coordinates (and the
C     tube coordinates below) in such an order that they form a single 
C     closed surface.
C
      rblen = real(fibdims(1))
      hbw = 0.5*real(fibdims(2))
      button(1) = hbw*(costheta + sintheta) + fibre(1)
      button(5) = hbw*(sintheta - costheta) + fibre(2)
      button(2) = hbw*(costheta - sintheta) + fibre(1)
      button(6) = hbw*(sintheta + costheta) + fibre(2)
      bl1 = rblen*costheta
      bl2 = rblen*sintheta
      button(3) = button(2) - bl1
      button(7) = button(6) - bl2
      button(4) = button(1) - bl1
      button(8) = button(5) - bl2
C
C     Next solve for the x,y positions of the corners of the tube.  The 
C     tube is defined all the way to the object location which is within
C     the button.  This doesn't really matter, but it makes the computation
C     easier.
C
      htw = 0.5*real(fibdims(3))
      tmp = htw*sintheta
      tube(1) = fibre(1) + tmp
      tube(2) = fibre(1) - tmp
      tube(3) = pivot(1) - tmp
      tube(4) = pivot(1) + tmp
      tmp = -htw*costheta
      tube(5) = fibre(2) + tmp
      tube(6) = fibre(2) - tmp
      tube(7) = pivot(2) - tmp
      tube(8) = pivot(2) + tmp
C
C     Exit
C
      end
