C+------------------------------------------------------------------------------
C
C     getval -- A set of subroutines for prompting and accepting 
C               input from the user.  The following routines are
C               included here:
C
C                  getval_r -- get a real value
C                  getval_i -- get an integer value
C                  getval_l -- get a logical value
C                  getval_c -- get a character value
C                  getval -- prompt for value and read in user response.
C                  decode_r -- decode a string into a real value
C                  decode_i -- decode a string into an integer value
C                  minmax_r -- decode a min, max or default into a real
C                  minmax_i -- decode a min, max or default into an integer
C
C-------------------------------------------------------------------------------

      subroutine getval_r(prompt,default,minstr,maxstr,val_r,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        getval_r
C
C     Invocation:
C        call getval_r(prompt,default,minstr,maxstr,val_r,status)
C
C     Description:
C        Prompt for and read in a real value from the user.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        prompt:  Character (>)
C           The basic prompt for the value
C        default:  Character (>)
C           The default value (string)
C        minstr:  Character (>)
C           The minimum allowable value (string).  N.B. This can be blank
C           if you don't wish to constrain the minimum value.
C        maxstr:  Character (>)
C           The maximum allowable value (string).  N.B. This can be blank
C           if you don't wish to constrain the maximum value.
C        val_r:  Real (<)
C           The output real value
C        status:  Integer (!)
C           Global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           decode_r
C           getval
C           minmax_r
C 
C        Library chr:
C           chr_len
C 
C     Algorithm:
C        -- Check and see if minimum, maximum and default values exist and if so
C           what they are.  Check that default is between min and max.
C        -- Loop until you get an acceptable answer
C        -- Form a prompt from the basic prompt and the default and 
C           write it out to the user.  Accept an answer as a character string
C        -- If the input string is '!!' then the user has requested an abort,
C           so we exit here with bad status.
C        -- If the input string has no non-blank characters, then use 
C           the default. If there was no default, send a message to that effect
C           and reprompt.  
C        -- If the user has replied with  something other than a blank value,
C           then try and decode the value.  If it won't decode, then reprompt.
C           If it decodes and is not between the specified min and max, then
C           reprompt.  Otherwise accept the value and exit the loop and the
C           routine with good status.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      character prompt*(*),default*(*),minstr*(*),maxstr*(*)
      integer status
      real val_r
C
C     Local variables
C
      logical ok,exmin,exmax,exdef
      real min_r,max_r,def_r
      character value*80
C
C     Function
C
      integer chr_len
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Look for existence of minimum, maximum and default values.
C
      call minmax_r(minstr,'minimum',exmin,min_r,status)
      call minmax_r(maxstr,'maximum',exmax,max_r,status)
      call minmax_r(default,'default',exdef,def_r,status)
      if (status .ne. af2_ok) go to 500
C
C     Check that default is between the min and max values.
C
      if (((exmin) .and. (def_r .lt. min_r)) .or. 
     :   ((exmax) .and. (def_r .gt. max_r))) then
         write(*,fmt='(a)') 'Default value not between min and max'//
     :   ' -- programming error'
         status = af2_err
         go to 500
      end if
C
C     Begin prompting loop.  Repeat until you either get an acceptable
C     value, there is an error or if there is a requested abort.
C
      ok = .false.
      do while (.not. ok)
C
C        First prompt the user and get his/her/its answer as a character
C
         call getval(prompt,default,value,status)
         if (status .ne. af2_ok) go to 500
C
C        If the user requests an abort, then get out of here with an 
C        error status
C
         if (value(1:2) .eq. '!!') then
            status = af2_err
            write(*,fmt='(a)') '***Abort requested***'
            go to 500
C
C        If the user reply was a blank, then accept the default. If the
C        default has no character size, then request another answer.
C
         else if (chr_len(value) .eq. 0) then
            if (exdef) then
               val_r = def_r
               ok = .true.
            else
               write(*,fmt='(a)') 'No default given -- try again'
            end if
C
C        If the user reply wasn't blank, try and decode it.  If it is an
C        invalid value, then send the user back around the loop
C
         else
            call decode_r(value,val_r,status)
            if (status .ne. af2_ok) then
               write(*,fmt='(a)') 'Invalid input -- try again'
               status = af2_ok
            else
               if (((val_r .lt. min_r) .and. (exmin)) .or. 
     :            ((val_r .gt. max_r) .and. (exmax))) then
                  write(*,fmt='(a,a,a,a,a)') 
     :               'Invalid input -- value must be between ',
     :               minstr(:chr_len(minstr)),' and ',
     :               maxstr(:chr_len(maxstr)),' -- try again'
               else
                  ok = .true.
               end if
            end if
         end if
      end do
  500 continue
      end

      subroutine getval_i(prompt,default,minstr,maxstr,val_i,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        getval_i
C
C     Invocation:
C        call getval_i(prompt,default,minstr,maxstr,val_i,status)
C
C     Description:
C        Prompt for and read in an integer value from the user.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        prompt:  Character (>)
C           The basic prompt for the value
C        default:  Character (>)
C           The default value (string)
C        minstr:  Character (>)
C           The minimum allowable value (string).  N.B. This can be blank
C           if you don't wish to constrain the minimum value.
C        maxstr:  Character (>)
C           The maximum allowable value (string).  N.B. This can be blank
C           if you don't wish to constrain the maximum value.
C        val_i:  Integer (<)
C           The output integer value
C        status:  Integer (!)
C           Global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           decode_i
C           getval
C           minmax_i
C 
C        Library chr:
C           chr_len
C 
C     Algorithm:
C        -- Check and see if minimum, maximum and default values exist and if so
C           what they are.  Check that default is between min and max.
C        -- Loop until you get an acceptable answer
C        -- Form a prompt from the basic prompt and the default and 
C           write it out to the user.  Accept an answer as a character string
C        -- If the input string is '!!' then the user has requested an abort,
C           so we exit here with bad status.
C        -- If the input string has no non-blank characters, then use 
C           the default. If there was no default, send a message to that effect
C           and reprompt.  
C        -- If the user has replied with  something other than a blank value,
C           then try and decode the value.  If it won't decode, then reprompt.
C           If it decodes and is not between the specified min and max, then
C           reprompt.  Otherwise accept the value and exit the loop and the
C           routine with good status.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      character prompt*(*),default*(*),minstr*(*),maxstr*(*)
      integer status,val_i
C
C     Local variables
C
      logical ok,exmin,exmax,exdef
      integer min_i,max_i,def_i
      character value*80
C
C     Function
C
      integer chr_len
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Look for existence of minimum, maximum and default values.
C
      call minmax_i(minstr,'minimum',exmin,min_i,status)
      call minmax_i(maxstr,'maximum',exmax,max_i,status)
      call minmax_i(default,'default',exdef,def_i,status)
      if (status .ne. af2_ok) go to 500
C
C     Check that default is between the min and max values.
C
      if (((exmin) .and. (def_i .lt. min_i)) .or. 
     :   ((exmax) .and. (def_i .gt. max_i))) then
         write(*,fmt='(a)') 'Default value not between min and max'//
     :   ' -- programming error'
         status = af2_err
         go to 500
      end if
C
C     Begin prompting loop.  Repeat until you either get an acceptable
C     value, there is an error or if there is a requested abort.
C
      ok = .false.
      do while (.not. ok)
C
C        First prompt the user and get his/her/its answer as a character
C
         call getval(prompt,default,value,status)
         if (status .ne. af2_ok) go to 500
C
C        If the user requests an abort, then get out of here with an 
C        error status
C
         if (value(1:2) .eq. '!!') then
            status = af2_err
            write(*,fmt='(a)') '***Abort requested***'
            go to 500
C
C        If the user reply was a blank, then accept the default. If the
C        default has no character size, then request another answer.
C
         else if (chr_len(value) .eq. 0) then
            if (exdef) then
               val_i = def_i
               ok = .true.
            else
               write(*,fmt='(a)') 'No default given -- try again'
            end if
C
C        If the user reply wasn't blank, try and decode it.  If it is an
C        invalid value, then send the user back around the loop
C
         else
            call decode_i(value,val_i,status)
            if (status .ne. af2_ok) then
               write(*,fmt='(a)') 'Invalid input -- try again'
               status = af2_ok
            else
               if (((val_i .lt. min_i) .and. (exmin)) .or. 
     :            ((val_i .gt. max_i) .and. (exmax))) then
                  write(*,fmt='(a,a,a,a,a)') 
     :               'Invalid input -- value must be between ',
     :               minstr(:chr_len(minstr)),' and ',
     :               maxstr(:chr_len(maxstr)),' -- try again'
               else
                  ok = .true.
               end if
            end if
         end if
      end do
  500 continue
      end

      subroutine getval_l(prompt,default,val_l,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        getval_l
C
C     Invocation:
C        call getval_l(prompt,default,val_l,status)
C
C     Description:
C        Prompt for and read in a logical value from the user.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        prompt:  Character (>)
C           The basic prompt for the value
C        default:  Character (>)
C           The default value (string)
C        val_l:  Logical (<)
C           The output logical value
C        status:  Integer (!)
C           Global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, do while, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           getval
C 
C        Library chr:
C           chr_lcase
C           chr_len
C 
C     Algorithm:
C        -- Check to make sure the default value is acceptable (if it exists).
C        -- Loop until you get an acceptable answer
C        -- Form a prompt from the basic prompt and the default and 
C           write it out to the user.  Accept an answer as a character string
C        -- If the input string is '!!' then the user has requested an abort,
C           so we exit here with bad status.
C        -- If the input string has no non-blank characters, then use 
C           the default. If there was no default, send a message to that effect
C           and reprompt.  
C        -- If the user has replied with something other than a blank value,
C           then first convert it to lower case.  Check that the lower case
C           answer is one of the following: 'y','n','yes' or 'no'.  Anything
C           else should cause a reprompt.   Otherwise accept the value and 
C           exit the loop and the routine with good status.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      character prompt*(*),default*(*)
      logical val_l
      integer status
C
C     Local variables
C
      logical ok,def_l,exdef
      character value*80
C
C     Function
C
      integer chr_len
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Check for default value.
C
      if (chr_len(default) .eq. 0) then
         exdef = .false.
         def_l = .true.
C
C     If it exists, then decode it
C
      else
         exdef = .true.
         if ((default(1:1) .eq. 'y') .or. (default(1:1) .eq. 'Y')) then
            def_l = .true.
         else if ((default(1:1) .eq. 'n') .or. 
     :      (default(1:1) .eq. 'N')) then
            def_l = .false.
         else
            write(*,fmt='(a)') 'Default value must be Y or N'//
     :         ' -- programming error'
            status = af2_err
            go to 500
         end if
      end if
C
C     Begin prompting loop.  Repeat until you either get an acceptable
C     value, there is an error or if there is a requested abort.
C
      ok = .false.
      do while (.not. ok)
C
C        First prompt the user and get his/her/its answer as a character
C
         call getval(prompt,default,value,status)
         if (status .ne. af2_ok) go to 500
C
C        If the user requests an abort, then get out of here with an 
C        error status
C
         if (value(1:2) .eq. '!!') then
            status = af2_err
            write(*,fmt='(a)') '***Abort requested***'
            go to 500
C
C        If the user reply was a blank, then accept the default. If the
C        default has no character size, then request another answer.
C
         else if (chr_len(value) .eq. 0) then
            if (exdef) then
               val_l = def_l
               ok = .true.
            else
               write(*,fmt='(a)') 'No default given -- try again'
            end if
C
C        If the user reply wasn't blank, convert it to lower case.  If it is an
C        invalid value, then send the user back around the loop.  Make sure
C        that we only accept the following: y,n,Y,N,yes,no,YES,NO
C
         else
            call chr_lcase(value)
C
C           Single letter response. Could be either 'y' or 'n'
C
            if (chr_len(value) .eq. 1) then
               if (value(1:1) .eq. 'y') then
                 val_l = .true.
                 ok = .true.
               else if (value(1:1) .eq. 'n') then
                  val_l = .false.
                  ok = .true.
               else
                  write(*,fmt='(a)') 'Invalid input -- try again'
               end if
C
C           Two letter response.  Can only be 'no'
C
            else if (chr_len(value) .eq. 2) then
               if (value(1:2) .eq. 'no') then
                  val_l = .false.
                  ok = .true.
               else
                  write(*,fmt='(a)') 'Invalid input -- try again'
               end if
C
C           Three letter response.  Can only be 'yes'
C
            else if (chr_len(value) .eq. 3) then
               if (value(1:3) .eq. 'yes') then
                  val_l = .true.
                  ok = .true.
               else
                  write(*,fmt='(a)') 'Invalid input -- try again'
               end if
C
C           Anything else is illegal
C
            else
               write(*,fmt='(a)') 'Invalid input -- try again'
            end if
         end if
      end do
C
C     Exit
C
  500 continue
      end

      subroutine getval_c(prompt,default,val_c,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        getval_c
C
C     Invocation:
C        call getval_c(prompt,default,val_c,status)
C
C     Description:
C        Prompt for and read in a character value from the user.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        prompt:  Character (>)
C           The basic prompt for the value
C        default:  Character (>)
C           The default value (string)
C        val_c:  Character (<)
C           The output character value
C        status:  Integer (!)
C           Global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           getval
C 
C        Library chr:
C           chr_len
C 
C     Algorithm:
C        -- Form a prompt from the basic prompt and the default and 
C           write it out to the user.  Accept an answer as a character string
C        -- If the input string is '!!' then the user has requested an abort,
C           so we exit here with bad status.
C        -- If the input string has no non-blank characters, then look the
C           accept the default as the answer.
C        -- If the user has replied with something other than a blank value,
C           then accept that as the answer.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      character prompt*(*),default*(*),val_c*(*)
      integer status
C
C     Local variables
C
      character value*80
C
C     Function
C
      integer chr_len
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     First prompt the user and get his/her/its answer as a character
C
      call getval(prompt,default,value,status)
      if (status .ne. af2_ok) go to 500
C
C     If the user requests an abort, then get out of here with an 
C     error status
C
      if (value(1:2) .eq. '!!') then
         status = af2_err
         write(*,fmt='(a)') '***Abort requested***'
         go to 500
C
C     If the user reply was a blank, then accept the default.
C
      else if (chr_len(value) .eq. 0) then
         val_c = default
C
C     If the user reply wasn't blank, accept it
C
      else
         val_c = value
      end if
  500 continue
      end

      subroutine getval(prompt,default,value,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        getval
C
C     Invocation:
C        call getval(prompt,default,value,status)
C
C     Description:
C        Prompt for and read in a generic value from the user.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        prompt:  Character (>)
C           The basic prompt for the value
C        default:  Character (>)
C           The default value (string)
C        value:  Character (<)
C           The output character value
C        status:  Integer (!)
C           Global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, $ format specifier, 
C        variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library chr:
C           chr_ldblk
C           chr_len
C 
C     Algorithm:
C        -- Make sure the basic prompt string and the default value string
C           aren't too big for the final prompting string.
C        -- Form a prompt from the basic prompt and the default and 
C           write it out to the user.  Accept an answer as a character string
C        -- Strip any leading blanks from the answer.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      character prompt*(*),default*(*),value*(*)
      integer status
C
C     Local variables
C
      integer len1,len2
C
C     Local parameters
C
      integer len3
      parameter (len3=80)
      character message*(len3)
C
C     Functions
C
      integer chr_len
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Get the length of the prompt and default strings and make sure
C     they aren't longer than the total allowed for the final prompt.
C
      len1 = chr_len(prompt)
      len2 = chr_len(default)
      if (len1 + len2 .gt. len3) then
         write(*,fmt='(a)') 
     :    'Prompt and default strings too long -- programming error'
         status = af2_err
         go to 500
      end if
C
C     If the default string has no size then don't include a default 
C     in the final prompt.  Otherwise include one enclosed in parenthesis.
C
      if (len2 .gt. 0) then
         message = prompt(:len1)//' ('//default(:len2)//') > '
      else
         message = prompt(:len1)//' > '
      end if
C
C     Now prompt the user and get the response as a character string.
C     Sorry, I know the dollar sign is a no-no, but I couldn't help myself...
C
      len1 = chr_len(message)
      write(*,fmt='(a,a,$)') message(:len1),' '
      read(*,fmt='(a)') value
C
C     Now strip any leading blanks from the value string before passing it
C     back to the calling routine...
C
      call chr_ldblk(value)
C
C     Exit
C
  500 continue
      end


      subroutine decode_r(string,val,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        decode_r
C
C     Invocation:
C        call decode_r(string,val,status)
C
C     Description:
C        Decode an real value from an input string
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        string:  Character (>)
C           The input string
C        val:  Real (<)
C           The output real value
C        status:  Integer (!)
C           Global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        none
C 
C     Algorithm:
C        -- Copy the input string to a temporary string.
C        -- Do an internal read of the string in free format into a real 
C           variable.  If this fails then send back a bad status.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      character string*(*)
      real val
      integer status
C
C     Local variables
C
      character string2*80
      integer iostat
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Copy the input string over and try to read a value from it
C
      string2 = string
      read(string2,fmt=*,iostat=iostat) val
C
C     If it didn't read properly, then signal an error
C
      if (iostat .ne. 0) status = af2_err
C
C     Exit
C
  500 continue
      end


      subroutine decode_i(string,val,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        decode_i
C
C     Invocation:
C        call decode_i(string,val,status)
C
C     Description:
C        Decode an integer value from an input string
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        string:  Character (>)
C           The input string
C        val:  Integer (<)
C           The output integer value
C        status:  Integer (!)
C           Global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        none
C 
C     Algorithm:
C        -- Copy the input string to a temporary string.
C        -- Do an internal read of the string in free format into an integer 
C           variable.  If this fails then send back a bad status.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      character string*(*)
      integer val
      integer status
C
C     Local variables
C
      character string2*80
      integer iostat
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Copy the input string over and try to read a value from it
C
      string2 = string
      read(string2,fmt=*,iostat=iostat) val
C
C     If it didn't read properly, then signal an error
C
      if (iostat .ne. 0) status = af2_err
C
C     Exit
C
  500 continue
      end


      subroutine minmax_r(string,type,exist,val_r,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        minmax_r
C
C     Invocation:
C        call minmax_r(string,type,exist,val_r,status)
C
C     Description:
C        Decode a real value from an input minimum, maximum or default
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        string:  Character (>)
C           The input string
C        type:  Character (>)
C           Type of input (minimum, maximum or default) for error messages
C        exist:  Logical (<)
C           TRUE if a value exists in the string
C        val_r:  Integer (<)
C           The output real value
C        status:  Integer (!)
C           Global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           decode_r
C 
C        Library chr:
C           chr_len
C 
C     Algorithm:
C        -- If the string has zero length, then the value doesn't exist
C        -- Decode the string.  If there is an error in decoding it, then
C           it is surely a programming error.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      character string*(*)
      character type*(*)
      logical exist
      real val_r
      integer status
C
C     Functions
C
      integer chr_len
C
C     First check that the string has a length.  If it doesn't then the
C     value doesn't exist.
C
      if (chr_len(string) .eq. 0) then
         exist = .false.
         val_r = 0.0
C
C     If it does exist, then decode it.  Check for errors in the decoding
C
      else
         call decode_r(string,val_r,status)
         if (status .ne. af2_ok) then
            write(*,fmt='(a,a,a)') 
     :         'Bad ',type,' value -- programming error'
            go to 500
         end if
         exist = .true.
      end if
C
C     Exit
C
  500 continue
      end


      subroutine minmax_i(string,type,exist,val_i,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        minmax_i
C
C     Invocation:
C        call minmax_i(string,type,exist,val_i,status)
C
C     Description:
C        Decode an integer value from an input minimum, maximum or default
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        string:  Character (>)
C           The input string
C        type:  Character (>)
C           Type of input (minimum, maximum or default) for error messages
C        exist:  Logical (<)
C           TRUE if a value exists in the string
C        val_i:  Integer (<)
C           The output integer value
C        status:  Integer (!)
C           Global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           decode_i
C 
C        Library chr:
C           chr_len
C 
C     Algorithm:
C        -- If the string has zero length, then the value doesn't exist
C        -- Decode the string.  If there is an error in decoding it, then
C           it is surely a programming error.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
C
C     Parameters
C
      character string*(*)
      character type*(*)
      logical exist
      integer val_i
      integer status
C
C     Functions
C
      integer chr_len
C
C     First check that the string has a length.  If it doesn't then the
C     value doesn't exist.
C
      if (chr_len(string) .eq. 0) then
         exist = .false.
         val_i = 0
C
C     If it does exist, then decode it.  Check for errors in the decoding
C
      else
         call decode_i(string,val_i,status)
         if (status .ne. af2_ok) then
            write(*,fmt='(a,a,a)') 
     :         'Bad ',type,' value -- programming error'
            go to 500
         end if
         exist = .true.
      end if
C
C     Exit
C
  500 continue
      end
