      subroutine platexy(ra,dec,eq,cra,cdec,ha,theta0,epobs, 
     :   scale0,scale1,scale2,retx,rety,first,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        platexy
C
C     Invocation:
C        call platexy(ra,dec,eq,cra,cdec,ha,theta0,epobs,scale0,
C           scale1,scale2,retx,rety,first,status)
C
C     Description:
C        Convert mean ra,dec (in radians) of an object to plate x,y 
C        coordinates (in mm) for autofib-2
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        ra:  Double precision (>)
C           Object Right Ascension (radians).
C        dec:  Double precision (>)
C           Object Declination (radians).
C        eq:  Double precision (>)
C           Object position equinox.
C        cra:  Double precision (>)
C           Right Ascension of the centre of the field (radians).
C        cdec:  Double precision (>)
C           Declination of the centre of the field (radians).
C        ha:  Double precision (>)
C           Apparent hour angle of field centre (hours).
C        theta0:  Double precision (>)
C           Plate rotation (radians)
C        epobs:  Double precision (>)
C           Epoch of observation (modified Julian day number).
C        scale0:  Double precision (>)
C           Plate scale coefficient: linear.
C        scale1:  Double precision (>)
C           Plate scale coefficient: non-linear.
C        scale2:  Double precision (>)
C           Plate scale coefficient: power.
C        retx:  Real (<)
C           X cartesian coordinate for robot position (mm).
C        rety:  Real (<)
C           Y cartesian coordinate for robot position (mm).
C        first:  Logical (!)
C           If first call to evaluate central refracted RA/Dec.
C        status:  Integer (!)
C           Global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           centrexy
C 
C        Library slalib:
C           sla_de2h
C           sla_dh2e
C           sla_ds2tp
C           sla_mapqkz
C           sla_refz
C 
C     Algorithm:
C        -- If this is the first time through the routine, then calculate
C           the observed place of the field centre.
C        -- Convert mean position to apparent place.
C        -- Convert apparent place to apparent azimuth and elevation.  From
C           the apparent elevation calculate the observed elevation from
C           the refraction correction.  Convert observed azimuth and elevation
C           to observed place.
C        -- Compute tangent plate coordinates (standard coordinates) from
C           the observed places of the object and the field centre.
C        -- Add in pincushion distortion and rotate the coordinates so that
C           they point along the cardinal axes of the instrument.
C
C     Authors:
C        (IRP): Ian R. Parry -- Durham
C        (TNW): Tim N. Wilkins -- Durham
C        (PCTR): Paul C.T. Rees -- La Palma
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        9-FEB-1987 (IRP):
C           Ian Parry's private version.
C        DEC-1993 (TNW):
C           Modified, mainly to simplify.
C        10-JUL-1995 (PCTR):
C           Check code and tidy up.
C        17-AUG-1995 (PCTR):
C           Use SLA_OBS to get an accurate position for the WHT.
C        10 Nov 1995 (JRL):
C           Small bug fix so that value of phi gets saved.  Farmed out 
C           work to find observed place of field centre to routine centrexy.
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global Parameters
C
      include 'AF2_PAR.INC'
      include 'AF2_MATH.INC'
C
C     Parameters
C
      double precision ra
      double precision dec
      double precision eq
      double precision cra
      double precision cdec
      double precision ha
      double precision theta0
      double precision epobs
      double precision scale0
      double precision scale1
      double precision scale2
      logical first
      real retx
      real rety
      integer status
C
C     Local variables:
C
      integer istat
      double precision alpha,az,cdecob,craob,decob,decap,el,eta,hao
      double precision ha_object,maprms(21),phi,r,raob,raap,refa,refb       
      double precision r2,theta1,theta,xi,zda,zdo,lst,height
C
C     Values to be saved
C
      save cdecob,craob,maprms,refa,refb,lst,phi
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     If this is the first pass through the subroutine, defined by FIRST 
C     being given the value .TRUE., get the position of the WHT and calculate 
C     the apparent position of the field centre. 
C
      if (first) then
         first = .false.
         call centrexy(cra,cdec,eq,epobs,ha,maprms,lst,phi,height,
     :      refa,refb,craob,cdecob)
      end if
C
C     Convert the given position to apparent place.
C
      call sla_mapqkz(ra,dec,maprms,raap,decap)
C
C     Convert the object apparent position to observed place.
C
      ha_object = lst - raap
      call sla_de2h(ha_object,decap,phi,az,el)
      zda = piover2 - el
      call sla_refz(zda,refa,refb,zdo)
      el = piover2 - zdo
      call sla_dh2e(az,el,phi,hao,decob)
      raob = lst - hao
C
C     Compute Xi and Eta.
C
      call sla_ds2tp(raob,decob,craob,cdecob,xi,eta,istat)
      if (istat .ne. 0) then
         write(*,'(A)') 
     :      'Error in spherical to tangent plane conversion.'
         status = af2_err
         go to 500
      end if
C
C     Convert to Polar Coordinates.
C
      r2 = xi*xi + eta*eta
      alpha = sqrt(r2)
C
C     Note: Autofib X increases to the west.
C
      if (r2 .ne. 0.0d+00) then
         theta = atan2(eta,-xi)
      else
         theta = 0.0d0
      end if
C
C     Convert from radians, allowing for a variable plate scale: rotate and 
C     convert to cartesian coordinates; flip the sign of Y to cope with WHT. 
C
      r = (alpha/scale0)*(1.0d+00 + scale1*alpha**scale2)
      theta1 = theta - theta0 + piover2
      retx = -real(r*cos(theta1))
      rety = -real(r*sin(theta1))
C
C     Exit
C
  500 continue
      end


      subroutine centrexy(cra,cdec,equinox,epobs,ha,maprms,lst,phi,
     :   height,refa,refb,cra_obs,cdec_obs)
C+------------------------------------------------------------------------------
C
C     Name:   
C        centrexy
C
C     Invocation:
C        call centrexy(cra,cdec,equinox,epobs,ha,maprms,lst,phi,
C           height,refa,refb,cra_obs,cdec_obs)
C
C     Description:
C        Convert mean ra,dec (in radians) of the plate centre to observed
C        ra,dec.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        cra:  Double precision (>)
C           Plate centre mean right ascension (radians).
C        cdec:  Double precision (>)
C           Plate centre mean declination (radians).
C        equniox:  Double precision (>)
C           Plate centre position equinox.
C        epobs:  Double precision (>)
C           Epoch of observation (modified Julian day number).
C        ha:  Double precision (>)
C           Apparent hour angle of field centre (hours).
C        maprms:  Double precision (<)
C           Parameters used by slalib to convert mean to apparent coordinates.
C        lst:  Double precision (<)
C           Local siderial time of observation (radians)
C        phi:  Double precision (<)
C           Latitude of observatory (radians)
C        height:  Double precision (<)
C           Height of the observatory (metres)
C        refa:  Double precision (<)
C           Refraction coefficient
C        refb:  Double precision (<)
C           Refraction coefficient
C        cra_obs:  Double precision (>)
C           Plate centre observed right ascension (radians).
C        cdec_obs:  Double precision (>)
C           Plate centre observed declination (radians).
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library slalib:
C           sla_de2h
C           sla_dh2e
C           sla_mappa
C           sla_mapqkz
C           sla_obs
C           sla_refco
C           sla_refz
C 
C     Algorithm:
C        -- Get the observatory location and height.
C        -- Calculate the transformation from mean to apparent coordinates
C           and transform the plate centre to apparent place.
C        -- Calculate the local siderial time as the sum of the input
C           hour-angle and the apparent ra of the field centre.
C        -- Calculate the refraction coefficients for the site based on
C           items in the af2_obs.inc file.
C        -- Convert apparent place to apparent azimuth and elevation.  From
C           the apparent elevation calculate the observed elevation from
C           the refraction correction.  Convert observed azimuth and elevation
C           to observed place.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        10 Nov 1995 (JRL):
C           Original version, most of which was removed from platexy.f
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_MATH.INC'
      include 'AF2_OBS.INC'
C
C     Parameters
C
      double precision cra
      double precision cdec
      double precision equinox
      double precision epobs
      double precision ha
      double precision maprms(*)
      double precision lst
      double precision phi
      double precision height
      double precision refa
      double precision refb
      double precision cra_obs
      double precision cdec_obs
C
C     Local variables
C
      integer nobs
      character obsnam*30
      double precision w,cra_ap,cdec_ap,ha_rad,az,el,zda,zdo,hao
C
C     Get latitude, longitude and height of telescope
C
      nobs = 0
      call sla_obs(nobs,oname,obsnam,w,phi,height)
C
C     Initialise mean <-> apparent transformation
C
      call sla_mappa(equinox,epobs,maprms)
C
C     Convert field centre position from mean to apparent place.
C
      call sla_mapqkz(cra,cdec,maprms,cra_ap,cdec_ap)
C
C     Calculate Local Siderial Time in radians
C
      ha_rad = ha*hourstorad
      lst = ha_rad + cra_ap
C
C     Calculate the local refraction coefficients
C
      call sla_refco(height,tdk,pmb,rh,wl,phi,tlr,eps,refa,refb)
C
C     Convert apparent hour angle and declination to azimuth and elevation
C
      call sla_de2h(ha_rad,cdec_ap,phi,az,el)
C
C     Calculate apparent zenith distance and refract it to give the observed
C     zenith distance
C
      zda = piover2 - el
      call sla_refz(zda,refa,refb,zdo)
C
C     Convert the observed azimuth and zenith distance to observed 
C     hour angle, ra and dec.
C
      el = piover2 - zdo
      call sla_dh2e(az,el,phi,hao,cdec_obs)
      cra_obs = lst - hao
C
C     Exit
C
      end

      subroutine xy2rd(x,y,equinox,cra,cdec,ha,sky_rot,epochobs,
     :   scale0,scale1,scale2,ra,dec,first,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        xy2rd
C
C     Invocation:
C        call xy2rd(x,y,equinox,cra,cdec,ha,sky_rot,epochobs,scale0,
C           scale1,scale2,ra,dec,first,status)
C
C     Description:
C        Plate x,y coordinates in mm to mean ra and dec.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        x:  Real (>)
C           X cartesian coordinate for robot position (mm)
C        y:  Real (>)
C           Y cartesian coordinate for robot position (mm)
C        equinox:  Double precision (>)
C           Object position equinox.
C        cra:  Double precision (>)
C           Right Ascension of the centre of the field (radians).
C        cdec:  Double precision (>)
C           Declination of the centre of the field (radians).
C        ha:  Double precision (>)
C           Apparent hour angle of field centre (hours).
C        sky_rot:  Double precision (>)
C           Plate rotation (radians)
C        epochobs:  Double precision (>)
C           Epoch of observation (modified Julian day number).
C        scale0:  Double precision (>)
C           Plate scale coefficient: linear.
C        scale1:  Double precision (>)
C           Plate scale coefficient: non-linear.
C        scale2:  Double precision (>)
C           Plate scale coefficient: power.
C        ra:  Double precision (<)
C           Object mean right ascension (radians).
C        dec:  Double precision (<)
C           Object mean declination (radians).
C        first:  Logical (!)
C           If first call to evaluate central refracted RA/Dec.
C        status:  Integer (!)
C           Global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           centrexy
C 
C        Library slalib:
C           sla_ampqk
C           sla_de2h
C           sla_dh2e
C           sla_dtp2s
C           sla_refro
C           sla_unpcd
C 
C     Algorithm:
C        -- If this is the first time through the routine, then calculate
C           the observed place of the field centre.
C        -- Convert the x,y to radians and then rotate them to the cardinal
C           axes.  Remove the pincushion distortion to yield the tangent 
C           plane (standard) coordinates.
C        -- Convert tangent plate coordinates to observed place using the
C           the observed place of the field centre
C        -- Convert observed place to azimuth and elevation and correct the
C           latter for refraction.  Convert corrected elevation and azimuth
C           back to ra,dec to give apparent place.
C        -- Convert apparent place to mean place.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        10 Nov 1995 (JRL):
C           Original version
C        13 May 1996 (JRL):
C           Added test to arguments of atan2 to make sure you never got a 
C           vector of length zero. 
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
      include 'AF2_MATH.INC'
      include 'AF2_OBS.INC'
C
C     Parameters
C
      real x
      real y
      double precision equinox
      double precision cra
      double precision cdec
      double precision ha
      double precision sky_rot
      double precision epochobs
      double precision scale0
      double precision scale1
      double precision scale2
      double precision ra
      double precision dec
      logical first
      integer status
C
C     Local variables
C
      double precision maprms(21),lst,phi,height,refa,refb
      double precision cra_obs,cdec_obs,x2,y2,theta1,theta
      double precision x3,y3,eta,xi,ra_obs,dec_obs,hao,az,el
      double precision zdo,dz,zda,haa,dec_app,ra_app,r
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     If this is the first pass through the subroutine, defined by FIRST 
C     being given the value .TRUE., get the position of the WHT and calculate 
C     the apparent position of the field centre. 
C
      if (first) then
         first = .false.
         call centrexy(cra,cdec,equinox,epochobs,ha,maprms,lst,phi,
     :      height,refa,refb,cra_obs,cdec_obs)
      end if
C
C     First convert x,y to radians
C
      x2 = dble(x)*scale0
      y2 = dble(y)*scale0
C
C     Now rotate the x,y coordinates back to the cardinal axes
C
      if ((x2**2 + y2**2) .gt. 0.0) then
         theta1 = atan2(-y2,-x2)
      else
         theta1 = 0.0
      end if
      theta = theta1 + sky_rot - piover2
      r = sqrt(x2**2 + y2**2)
      x3 = r*cos(theta)
      y3 = r*sin(theta)
C
C     Remove the pincushion distortion.  Flip sign of xi since autofib X 
C     increases to the West.
C
      call sla_unpcd(scale1,x3,y3)
      eta = y3
      xi = -x3
C
C     Convert tangent plane coodinates to observed place
C
      call sla_dtp2s(xi,eta,cra_obs,cdec_obs,ra_obs,dec_obs)
C
C     Convert the observed place to azimuth and elevation
C
      hao = lst - ra_obs
      call sla_de2h(hao,dec_obs,phi,az,el)
C
C     Compute the refraction for the given zenith distance and add it 
C     into the elevation.
C
      zdo = piover2 - el
      call sla_refro(zdo,height,tdk,pmb,rh,wl,phi,tlr,eps,dz)
      zda = zdo + dz
      el = piover2 - zda
C
C     Now convert azimuth and elevation to apparent position
C
      call sla_dh2e(az,el,phi,haa,dec_app)
      ra_app = lst - haa
C
C     Finally, convert the apparent position to mean 
C
      call sla_ampqk(ra_app,dec_app,maprms,ra,dec)
C
C     Exit
C
  500 continue
      end
