      logical function rect_check(rect1,rect2)
C+------------------------------------------------------------------------------
C
C     Name:   
C        rect_check
C
C     Invocation:
C        overlap = rect_check(rect1,rect2)
C
C     Description:
C        Determine whether two rectangles overlap each other
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        rect1:  Real array (>)
C           The coordinates of the corners of the first rectangle.  The
C           first four elements are the X coordinates and the second four
C           are the Y.
C        rect2:  Real array (>)
C           The coordinates of the corners of the second rectangle.  The
C           first four elements are the X coordinates and the second four
C           are the Y.
C
C     Returns:
C        overlap:  Logical
C           TRUE if the rectangles overlap
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        none
C 
C     Algorithm:
C        -- Find the position angles of the rectangles w.r.t. the X axis and
C           calculate the sine and cosine of each
C        -- Rotate the rectangles about the origin by the position angle of
C           the first rectangle to form the coordinates X',Y'.  If the range 
C           of the coordinates in either X' or Y' doesn't overlap, then 
C           the rectangles don't overlap. If they don't overlap then exit 
C           with a false value.
C        -- If the ranges in both X' and Y' overlap, then rotate the original
C           X,Y to the position angle of the second rectangle to form the
C           coordinates X'',Y''. Test the ranges of the rotated coordinates as
C           before.  If there is one in which there is no overlap, then the
C           rectangles don't overlap and exit with a false value.
C        -- If you are unable to find one set of coordinates which don't 
C           overlap, then the rectangles overlap and exit with a true value.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      real rect1(8),rect2(8)
C   
C     Local variables
C
      logical overlap
      real trec1(8),trec2(8),dx,dy,sintheta(2),costheta(2)
      real minx1,minx2,maxx1,maxx2,miny1,miny2,maxy1,maxy2,rad
      integer i,j
C
C     Initialise the output variable.  The default is that the rectangles
C     do overlap
C
      overlap = .true.
C
C     First calculate the cosine and sine of the  angles in question.  
C     These are the position angles of the buttons w.r.t. to the x axis
C
      dx = rect1(4) - rect1(1)
      dy = rect1(8) - rect1(5)
      rad = sqrt(dx**2 + dy**2)
      costheta(1) = dx/rad
      sintheta(1) = dy/rad
      dx = rect2(4) - rect2(1)
      dy = rect2(8) - rect2(5)
      rad = sqrt(dx**2 + dy**2)
      costheta(2) = dx/rad
      sintheta(2) = dy/rad
C
C     Now for each position angle calculate a set of transformed coordinates.
C
      do 30 i = 1,2
C
C        First look at X only
C
         do 10 j = 1,4
            trec1(j) = rect1(j)*costheta(i) + rect1(j+4)*sintheta(i)
            trec2(j) = rect2(j)*costheta(i) + rect2(j+4)*sintheta(i)
  10     continue
C
C        Ok...Check the domain of the corners of each rectangle. 
C
         maxx1 = max(trec1(1),trec1(2),trec1(3),trec1(4))
         minx2 = min(trec2(1),trec2(2),trec2(3),trec2(4))
         if (maxx1 .lt. minx2) then
            overlap = .false.
            go to 500
         end if
         maxx2 = max(trec2(1),trec2(2),trec2(3),trec2(4))
         minx1 = min(trec1(1),trec1(2),trec1(3),trec1(4))
         if (maxx2 .lt. minx1) then
            overlap = .false.
            go to 500
         end if
C
C        Now look at Y only
C
         do 20 j = 1,4
            trec1(j+4) = -rect1(j)*sintheta(i) + rect1(j+4)*costheta(i)
            trec2(j+4) = -rect2(j)*sintheta(i) + rect2(j+4)*costheta(i)
  20     continue
C
C        Check the range of the corners of each rectangle.
C
         maxy1 = max(trec1(5),trec1(6),trec1(7),trec1(8))
         miny2 = min(trec2(5),trec2(6),trec2(7),trec2(8))
         if (maxy1 .lt. miny2) then
            overlap = .false.
            go to 500
         end if
         maxy2 = max(trec2(5),trec2(6),trec2(7),trec2(8))
         miny1 = min(trec1(5),trec1(6),trec1(7),trec1(8))
         if (maxy2 .lt. miny1) then
            overlap = .false.
            go to 500
         end if
  30  continue
C
C     Exit
C
  500 continue
      rect_check = overlap
      end
