      subroutine telescopeinfo(dir,field_size,scales,status)
C+------------------------------------------------------------------------------
C
C     Name:   
C        telescopeinfo
C
C     Invocation:
C        call telescopeinfo(dir,field_size,scales,status)
C
C     Description:
C        Read the telescope information file
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        dir:  Character (>)
C           The directory path to the telescope information file
C        field_size:  Real (<)
C           The radius of the accessable field
C        scales:  Real array (<)
C           The field distortion coefficients
C        status:  Integer (!)
C           The global status variable
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        include, implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        41
C
C     External routines called:
C        Library subaf2_configure:
C           readreal
C
C        Library chr:
C           chr_len
C 
C     Algorithm:
C        -- Open the telescope information file.  The name is in the OBS
C           include file and is prefixed with the relevant directory path.
C        -- Read the field diameter, the plate scale, the second order term
C           and the second order power term.
C        -- Close the telescope information file.
C        -- Convert the field diameter in mm to field radius in microns and 
C           convert the plate scale from arcsec/mm to radians/mm
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Global parameters
C
      include 'AF2_PAR.INC'
      include 'AF2_MATH.INC'
      include 'AF2_OBS.INC'
C
C     Parameters
C
      real field_size
      real scales(3)
      character dir*(*)
      integer status
C 
C     Local parameters
C
      integer fd
      parameter (fd=41)
C
C     Local variables
C
      integer len1,nread,iostat
      character*80 fname
C
C     Functions
C
      integer chr_len
C
C     Inherited status
C
      if (status .ne. af2_ok) go to 500
C
C     Section to open the file with the description of the telescope parameters.
C     Form full path name of file by combining the directory and root 
C     file name.  Attempt to open the file.  If it doesn't exist then
C     signal an error.
C
      len1 = chr_len(dir)
      fname = dir(:len1)//tif
      len1 = chr_len(fname)
      open(unit=fd,file=fname(:len1),status='old',access='sequential',
     :   iostat=iostat)
      if (iostat .ne. 0) then
         write(*,fmt='(a,a)') 'Error opening ',fname(:len1)
         status = af2_err
         go to 500
      end if
C
C     Now read the values from the file
C 
C     Field diameter in mm
C
      nread = 0
      call readreal(fd,nread,field_size,status)
C 
C     Plate scale in arc sec/mm
C
      call readreal(fd,nread,scales(1),status)
C 
C     Second order term
C
      call readreal(fd,nread,scales(2),status)
C 
C     Power for second order term
C
      call readreal(fd,nread,scales(3),status)
      if (status .ne. af2_ok) go to 500
C
C     Close the input file
C
      close(fd)
C 
C     Convert field diameter (mm) to field radius (microns)
C
      field_size = field_size*1000.0/2.0
C 
C     Convert scale(1) (plate scale) from arc sec/mm to radians/mm.
C
      scales(1) = scales(1)*real(degtorad)/3600.0
C
C     Exit
C
  500 continue
      end
