      SUBROUTINE sla_REFRO (ZOBS, HM, TDK, PMB, RH, WL, PHI, TLR,
     :                      EPS, REF)
*+
*     - - - - - -
*      R E F R O
*     - - - - - -
*
*  Atmospheric refraction for radio and optical wavelengths
*
*  Given:
*    ZOBS    d  observed zenith distance of the source (radian)
*    HM      d  height of the observer above sea level (metre)
*    TDK     d  ambient temperature at the observer (deg K)
*    PMB     d  pressure at the observer (millibar)
*    RH      d  relative humidity at the observer (range 0-1)
*    WL      d  effective wavelength of the source (micrometre)
*    PHI     d  latitude of the observer (radian, astronomical)
*    TLR     d  temperature lapse rate in the troposphere (degK/metre)
*    EPS     d  precision required to terminate iteration (radian)
*
*  Returned:
*    REF     d  refraction: in vacuo ZD minus observed ZD (radian)
*
*  Suggested values for the TLR and EPS arguments are 0.0065D0
*  and 1D-8 respectively.  The result is usually at least two
*  orders of magnitude more precise than the supplied EPS value.
*
*  This routine computes the refraction for zenith distances up to
*  and a little beyond 90 deg using the method of Hohenkerk and
*  Sinclair (NAO Technical Notes 59 and 63).  The code is a
*  modified form of the optical refraction subroutine AREF of
*  C.Hohenkerk (HMNAO, September 1984), with extensions to support
*  radio wavelengths.  Most of the modifications to the HMNAO optical
*  algorithm are cosmetic, but the numerical integration phase has
*  been rearranged for extra clarity;  in addition the angle arguments
*  have been changed to radians, any value of ZOBS is allowed, and other
*  argument values have been limited to safe values.  The radio
*  expressions were devised by A.T.Sinclair (RGO - private
*  communication), based upon the Essen & Froome refractivity formula
*  adopted in Resolution 1 of the 13th International Geodesy Association
*  General Assembly (Bulletin Geodesique 1963 p390).
*
*  The radio refraction is chosen by specifying WL > 100 micrometres.
*
*  Before use, the value of ZOBS is expressed in the range +/- pi.
*  If this ranged ZOBS is -ve, the result REF is computed from its
*  absolute value before being made -ve to match.  In addition, if
*  it has an absolute value greater than 93 deg, a fixed REF value
*  equal to the result for ZOBS = 93 deg is returned, appropriately
*  signed.
*
*  Fixed values of the water vapour exponent, height of tropopause, and
*  height at which refraction is negligible are used.
*
*  Called:  sla_DRANGE, sla__ATMT, sla__ATMS
*
*  Original version by C.Hohenkerk, HMNAO, September 1986.
*  This Starlink version by P.T.Wallace, 30 December 1994.
*-

      IMPLICIT NONE

      DOUBLE PRECISION ZOBS,HM,TDK,PMB,RH,WL,PHI,TLR,EPS,REF

*
*  Fixed parameters
*
      DOUBLE PRECISION D93,GCR,DMD,DMW,S,DELTA,HT,HS
*  93 degrees in radians
      PARAMETER (D93=1.623156204D0)
*  Universal gas constant
      PARAMETER (GCR=8314.36D0)
*  Molecular weight of dry air
      PARAMETER (DMD=28.966D0)
*  Molecular weight of water vapour
      PARAMETER (DMW=18.016D0)
*  Mean Earth radius (metre)
      PARAMETER (S=6378120D0)
*  Exponent of temperature dependence of water vapour pressure
      PARAMETER (DELTA=18.36D0)
*  Height of tropopause (metre)
      PARAMETER (HT=11000D0)
*  Upper limit for refractive effects (metre)
      PARAMETER (HS=80000D0)

      INTEGER IS,K,N,I,J
      LOGICAL OPTIC,LOOP
      DOUBLE PRECISION ZOBS1,ZOBS2,HMOK,TDKOK,PMBOK,RHOK,WLOK,ALPHA,
     :                 TOL,WLSQ,GB,A,GAMAL,GAMMA,GAMM2,DELM2,PW0,W,
     :                 C1,C2,C3,C4,C5,C6,R0,TEMPO,DN0,DNDR0,SK0,F0,
     :                 RT,TT,DNT,DNDRT,SINE,ZT,FT,DNTS,DNDRTS,ZTS,FTS,
     :                 RS,DNS,DNDRS,ZS,FS,REFOLD,Z0,ZRANGE,FB,FF,FO,FE,
     :                 H,R,SZ,RG,DR,TG,DN,DNDR,T,F,REFP,REFT

      DOUBLE PRECISION sla_DRANGE

*  The refraction integrand
      DOUBLE PRECISION REFI
      REFI(R,DN,DNDR) = R*DNDR/(DN+R*DNDR)



*  Transform ZOBS into the normal range
      ZOBS1 = sla_DRANGE(ZOBS)
      ZOBS2 = MIN(ABS(ZOBS1),D93)

*  Keep other arguments within safe bounds
      HMOK = MIN(MAX(HM,-1D3),10D3)
      TDKOK = MIN(MAX(TDK,100D0),500D0)
      PMBOK = MIN(MAX(PMB,0D0),10000D0)
      RHOK = MIN(MAX(RH,0D0),1D0)
      WLOK = MAX(WL,0.1D0)
      ALPHA = MIN(MAX(ABS(TLR),0.001D0),0.01D0)

*  Tolerance for iteration
      TOL = MIN(ABS(EPS),0.1D0)/2D0

*  Decide whether optical or radio case - switch at 100 micron
      OPTIC = WLOK.LE.100D0

*  Set up model atmosphere parameters defined at the observer
      WLSQ = WLOK*WLOK
      GB = 9.784D0*(1D0-0.0026D0*COS(PHI+PHI)-0.00000028D0*HMOK)
      IF (OPTIC) THEN
         A = (287.604D0+(1.6288D0+0.0136D0/WLSQ)/WLSQ)
     :                                              *273.15D-6/1013.25D0
      ELSE
         A = 77.624D-6
      END IF
      GAMAL = (GB*DMD)/GCR
      GAMMA = GAMAL/ALPHA
      GAMM2 = GAMMA-2D0
      DELM2 = DELTA-2D0
      PW0 = RHOK*(TDKOK/247.1D0)**DELTA
      W = PW0*(1D0-DMW/DMD)*GAMMA/(DELTA-GAMMA)
      C1 = A*(PMBOK+W)/TDKOK
      IF (OPTIC) THEN
         C2 = (A*W+11.2684D-6*PW0)/TDKOK
      ELSE
         C2 = (A*W+12.92D-6*PW0)/TDKOK
      END IF
      C3 = (GAMMA-1D0)*ALPHA*C1/TDKOK
      C4 = (DELTA-1D0)*ALPHA*C2/TDKOK
      IF (OPTIC) THEN
         C5 = 0D0
         C6 = 0D0
      ELSE
         C5 = 371897D-6*PW0/TDKOK
         C6 = C5*DELM2*ALPHA/(TDKOK*TDKOK)
      END IF

*  At the observer
      R0 = S+HMOK
      CALL sla__ATMT(R0,TDKOK,ALPHA,GAMM2,DELM2,C1,C2,C3,C4,C5,C6,
     :                                               R0,TEMPO,DN0,DNDR0)
      SK0 = DN0*R0*SIN(ZOBS2)
      F0 = REFI(R0,DN0,DNDR0)

*  At the tropopause in the troposphere
      RT = S+HT
      CALL sla__ATMT(R0,TDKOK,ALPHA,GAMM2,DELM2,C1,C2,C3,C4,C5,C6,
     :                                                  RT,TT,DNT,DNDRT)
      SINE = SK0/(RT*DNT)
      ZT = ATAN2(SINE,SQRT(MAX(1D0-SINE*SINE,0D0)))
      FT = REFI(RT,DNT,DNDRT)

*  At the tropopause in the stratosphere
      CALL sla__ATMS(RT,TT,DNT,GAMAL,RT,DNTS,DNDRTS)
      SINE = SK0/(RT*DNTS)
      ZTS = ATAN2(SINE,SQRT(MAX(1D0-SINE*SINE,0D0)))
      FTS = REFI(RT,DNTS,DNDRTS)

*  At the stratosphere limit
      RS = S+HS
      CALL sla__ATMS(RT,TT,DNT,GAMAL,RS,DNS,DNDRS)
      SINE = SK0/(RS*DNS)
      ZS = ATAN2(SINE,SQRT(MAX(1D0-SINE*SINE,0D0)))
      FS = REFI(RS,DNS,DNDRS)

*
*  Integrate the refraction integral in two parts;  first in the
*  troposphere (K=1), then in the stratosphere (K=2)
*

*  Initialize previous refraction to ensure at least two iterations
      REFOLD = 1D6

*  Start off with 8 strips for the troposphere integration, and then
*  use the final troposphere value for the stratosphere integration,
*  which tends to need more strips
      IS = 8

*  Troposphere then stratosphere
      DO K = 1,2

*     Start Z, Z range, and start and end values
         IF (K.EQ.1) THEN
            Z0 = ZOBS2
            ZRANGE = ZT-Z0
            FB = F0
            FF = FT
         ELSE
            Z0 = ZTS
            ZRANGE = ZS-Z0
            FB = FTS
            FF = FS
         END IF

*     Sums of odd and even values
         FO = 0D0
         FE = 0D0

*     First time through loop we have to do every point
         N = 1

*     Start of iteration loop (terminates at specified precision)
         LOOP = .TRUE.
         DO WHILE (LOOP)

*        Strip width
            H = ZRANGE/DBLE(IS)

*        Initialize distance from Earth centre for quadrature pass
            IF (K.EQ.1) THEN
               R = R0
            ELSE
               R = RT
            END IF

*        One pass (no need to compute evens after first time)
            DO I=1,IS-1,N

*           Sine of observed zenith distance
               SZ = SIN(Z0+H*DBLE(I))

*           Find R (to nearest metre, maximum four iterations)
               IF (SZ.GT.1D-20) THEN
                  W = SK0/SZ
                  RG = R
                  DR = 1D6
                  J = 0
                  DO WHILE (ABS(DR).GT.1D0.AND.J.LT.4)
                     J=J+1
                     IF (K.EQ.1) THEN
                        CALL sla__ATMT(R0,TDKOK,ALPHA,GAMM2,DELM2,
     :                                  C1,C2,C3,C4,C5,C6,RG,TG,DN,DNDR)
                     ELSE
                        CALL sla__ATMS(RT,TT,DNT,GAMAL,RG,DN,DNDR)
                     END IF
                     DR = (RG*DN-W)/(DN+RG*DNDR)
                     RG = RG-DR
                  END DO
                  R = RG
               END IF

*           Find refractive index and integrand at R
               IF (K.EQ.1) THEN
                  CALL sla__ATMT(R0,TDKOK,ALPHA,GAMM2,DELM2,
     :                                    C1,C2,C3,C4,C5,C6,R,T,DN,DNDR)
               ELSE
                  CALL sla__ATMS(RT,TT,DNT,GAMAL,R,DN,DNDR)
               END IF
               F = REFI(R,DN,DNDR)

*           Accumulate odd and (first time only) even values
               IF (N.EQ.1.AND.MOD(I,2).EQ.0) THEN
                  FE = FE+F
               ELSE
                  FO = FO+F
               END IF
            END DO

*        Evaluate the integrand using Simpson's Rule
            REFP = H*(FB+4D0*FO+2D0*FE+FF)/3D0

*        Has the required precision been achieved?
            IF (ABS(REFP-REFOLD).GT.TOL) THEN

*           No: prepare for next iteration

*           Save current value for convergence test
               REFOLD = REFP

*           Double the number of strips
               IS = IS+IS

*           Sum of all current values = sum of next pass's even values
               FE = FE+FO

*           Prepare for new odd values
               FO = 0D0

*           Skip even values next time
               N = 2
            ELSE

*           Yes: save troposphere component and terminate loop
               IF (K.EQ.1) REFT = REFP
               LOOP = .FALSE.
            END IF
         END DO
      END DO

*  Result
      REF = REFT+REFP
      IF (ZOBS1.LT.0D0) REF = -REF

      END
